// SPACK.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the code for the SPACK user interface form, as well as the HTML
// authoring code - in other words, all of the interesting stuff is here.

#region Using declarations

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;

#endregion

namespace SPACK_NS {

	public partial class SPACK : Form {

		#region Variables, initialization code, and Main

		internal SPACK_Options_Set options_set = new SPACK_Options_Set();
		private string strSelectedPhoto = "";
		private string strViewingPhoto = "";
		private int iLoadedPhoto = 0, iLastLoadedPhoto = 0;
		private Thread threadLoadImages;
		private Image imagePreview = null;
		private string strAlbumFilename = "";
		internal string strStatus = "";
		internal bool bCreatingAlbum = false;
		private SPACK_Report report = null;
		private bool bChanged = false;

		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.Run(new SPACK(System.Environment.GetCommandLineArgs()));
		}

		public SPACK(string[] strCommandLineArguments) {
			InitializeComponent();
			if (strCommandLineArguments.Length >= 2)  // if we've been passed the name of a file, save it so that we can load it later.
				strAlbumFilename = strCommandLineArguments[1];
		}


		#endregion

		#region Button-click event handlers

		private void btnAddPhotos_Click(object sender, EventArgs e) {
			OpenFileDialog o = new OpenFileDialog();
			o.Filter = "Images (*.jpg; *.jpeg; *.gif; *.bmp; *.tiff; *.targa; *.tga)|*.jpg;*.jpeg;*.gif;*.bmp;*.tiff;*.targa;*.tga|All Files (*.*)|*.*";
			o.Multiselect = true;
			if (System.IO.Directory.Exists(txtBaseFolder.Text))
				o.InitialDirectory = txtBaseFolder.Text;
			if (o.ShowDialog() == DialogResult.OK) {
				lvPhotos.BeginUpdate();
				foreach (string strFileName in o.FileNames) {
					string strAmendedFileName = strFileName;
					if (strAmendedFileName.StartsWith(txtBaseFolder.Text))
						strAmendedFileName = strAmendedFileName.Replace(txtBaseFolder.Text, "");
					lvPhotos.Items.Add(new ListViewItem(new string[] { strAmendedFileName, "" }));
				}
				lvPhotos.EndUpdate();
				bChanged = true;
			}
		}

		private void btnInsertPageBreak_Click(object sender, EventArgs e) {
			if (lvPhotos.SelectedItems.Count == 1)
				lvPhotos.Items.Insert(lvPhotos.Items.IndexOf(lvPhotos.SelectedItems[0]), new ListViewItem(new string[] { "{Page Start}", "" }));
			else
				lvPhotos.Items.Add(new ListViewItem(new string[] { "{Page Start}", "" }));
			bChanged = true;
		}

		private void btnMoveUp_Click(object sender, EventArgs e) {
			lvPhotos.BeginUpdate();
			ArrayList arrayItems = new ArrayList();
			// take all photos out of list, and create array with new positions
			while (lvPhotos.SelectedItems.Count > 0) {
				ListViewItem item = lvPhotos.SelectedItems[lvPhotos.SelectedItems.Count - 1];
				arrayItems.Insert(0, new SortedItem(Math.Max(0, lvPhotos.Items.IndexOf(item) - 1), item));
				lvPhotos.Items.Remove(item);
			}
			// reinsert items at new indices
			while (arrayItems.Count > 0) {
				lvPhotos.Items.Insert(((SortedItem) arrayItems[0]).iNewIndex, ((SortedItem) arrayItems[0]).item);
				arrayItems.RemoveAt(0);
			}
			lvPhotos.EndUpdate();
			bChanged = true;
		}

		private void btnMoveDown_Click(object sender, EventArgs e) {
			int iTotalItems = lvPhotos.Items.Count;
			lvPhotos.BeginUpdate();
			ArrayList arrayItems = new ArrayList();
			// take all photos out of list, and create array with new positions
			while (lvPhotos.SelectedItems.Count > 0) {
				ListViewItem item = lvPhotos.SelectedItems[lvPhotos.SelectedItems.Count - 1];
				arrayItems.Insert(0, new SortedItem(lvPhotos.Items.IndexOf(item) + 1, item));
				lvPhotos.Items.Remove(item);
			}
			// reinsert items at new indices
			while (arrayItems.Count > 0) {
				lvPhotos.Items.Insert(Math.Min(lvPhotos.Items.Count, ((SortedItem) arrayItems[0]).iNewIndex), ((SortedItem) arrayItems[0]).item);
				arrayItems.RemoveAt(0);
			}
			lvPhotos.EndUpdate();
			bChanged = true;
		}

		private void btnRemove_Click(object sender, EventArgs e) {
			lvPhotos.BeginUpdate();
			while (lvPhotos.SelectedItems.Count > 0) {
				lvPhotos.Items.Remove(lvPhotos.SelectedItems[0]);
				bChanged = true;
			}
			lvPhotos.EndUpdate();
			btnRemove.Enabled = btnMoveDown.Enabled = btnMoveUp.Enabled = false;
			txtPhotoCaption.Text = "";
			txtPhotoCaption.Enabled = false;
		}

		private void btnBrowseBasePath_Click(object sender, EventArgs e) {
			FolderBrowserDialog f = new FolderBrowserDialog();
			if (f.ShowDialog() == DialogResult.OK)
				txtBaseFolder.Text = f.SelectedPath + System.IO.Path.DirectorySeparatorChar;
		}

		#endregion

		#region Menu Options

		#region File Menu

		private void menuFileNewAlbum_Click(object sender, EventArgs e) {
			if (bChanged == true) {
				if (MessageBox.Show("The current album has changed since it was last saved. Continuing will cause these changes to be lost. Do you wish to continue?", "Confirm New Album", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1) == DialogResult.OK)
					bChanged = false;
			}
			if (bChanged == false)
				CreateNewAlbum();
		}

		private void menuFileLoadAlbum_Click(object sender, EventArgs e) {
			if (bChanged == true) {
				if (MessageBox.Show("The current album has changed since it was last saved. Continuing will cause these changes to be lost. Do you wish to continue?", "Confirm New Album", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1) == DialogResult.OK)
					bChanged = false;
			}
			if (bChanged == false) {
				OpenFileDialog open = new OpenFileDialog();
				open.Multiselect = false;
				open.Filter = "SPACK Albums (*.spack)|*.spack";
				if (open.ShowDialog() == DialogResult.OK)
					LoadAlbum(open.FileName);
			}
		}

		private void menuFileSaveAlbum_Click(object sender, EventArgs e) {
			if (strAlbumFilename == "") {
				SaveFileDialog save = new SaveFileDialog();
				save.Filter = "SPACK Albums (*.spack)|*.spack";
				// find default filename
				string strDefaultFilename = (txtAlbumTitle.Text.Length > 0 ? txtAlbumTitle.Text : "New SPACK Album");
				string strDefaultFilenameWithExtension = strDefaultFilename + ".spack";
				for (int iTryAgain = 1; System.IO.File.Exists(strDefaultFilenameWithExtension) == true; iTryAgain++)
					strDefaultFilenameWithExtension = strDefaultFilename + " (" + iTryAgain + ").spack";
				save.FileName = strDefaultFilenameWithExtension;
				if (save.ShowDialog() == DialogResult.OK)
					SetFilename(save.FileName);
			}
			if (strAlbumFilename != "") {
				SaveAlbum(strAlbumFilename);
				bChanged = false;
			}
		}

		private void menuFileSaveAlbumAs_Click(object sender, EventArgs e) {
			SaveFileDialog save = new SaveFileDialog();
			save.Filter = "SPACK Albums (*.spack)|*.spack";
			// find default filename
			string strDefaultFilename = (txtAlbumTitle.Text.Length > 0 ? txtAlbumTitle.Text : "New SPACK Album");
			string strDefaultFilenameWithExtension = strDefaultFilename + ".spack";
			for (int iTryAgain = 1; System.IO.File.Exists(strDefaultFilenameWithExtension) == true; iTryAgain++)
				strDefaultFilenameWithExtension = strDefaultFilename + " (" + iTryAgain + ").spack";
			save.FileName = strDefaultFilenameWithExtension;
			if (save.ShowDialog() == DialogResult.OK) {
				SetFilename(save.FileName);
				SaveAlbum(strAlbumFilename);
				bChanged = false;
			}
		}

		private void menuFileWriteAlbum_Click(object sender, EventArgs e) {
			FolderBrowserDialog folder = new FolderBrowserDialog();
			if (folder.ShowDialog() == DialogResult.OK)
				CreateAlbum(folder.SelectedPath);
		}

		private void menuFileProgramOptions_Click(object sender, EventArgs e) {
			SPACK_Options options = new SPACK_Options(options_set);
			if (options.ShowDialog() == DialogResult.OK)
				options_set = options.options_set;
		}

		private void menuFileExit_Click(object sender, EventArgs e) {
			if (bChanged == true) {
				if (MessageBox.Show("The current album has changed since it was last saved. Continuing will cause these changes to be lost. Do you wish to continue?", "Confirm New Album", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1) == DialogResult.OK)
					bChanged = false;
			}
			if (bChanged == false)
				this.Close();
		}

		#endregion

		#region Album Menu

		private void menuAlbumSortPhotosByName_Click(object sender, EventArgs e) {
			SortPhotos("Name");
		}

		private void menuAlbumSortPhotosByDateCreated_Click(object sender, EventArgs e) {
			SortPhotos("Date Created");
		}

		private void menuAlbumSortPhotosByDateModified_Click(object sender, EventArgs e) {
			SortPhotos("Date Modified");
		}

		#endregion

		#region Help Menu

		private void menuHelpHelp_Click(object sender, EventArgs e) {
			if (File.Exists("SPACK.chm"))
				Help.ShowHelp(this, "SPACK.chm");
		}

		private void menuHelpAbout_Click(object sender, EventArgs e) {
			new SPACK_About().ShowDialog();
		}

		#endregion

		#endregion

		#region Other event handlers

		private void SPACK_Load(object sender, EventArgs e) {
			options_set = SPACK_Options.LoadOptions();
			lvPhotos.Items.Add(new ListViewItem(new string[] { "{Page Start}", "" }));
			threadLoadImages = new Thread(new ThreadStart(LoadImagesThread));
			threadLoadImages.IsBackground = true;
			threadLoadImages.Start();
			if (strAlbumFilename.Length > 0)
				LoadAlbum(strAlbumFilename);
			else
				CreateNewAlbum();
		}

		private void SPACK_Resize(object sender, EventArgs e) {
			columnHeaderCaption.Width = columnHeaderFilename.Width = (groupPhotos.Width - 40) / 2;
		}

		private void lvPhotos_SelectedIndexChanged(object sender, EventArgs e) {
			strSelectedPhoto = "";
			if (lvPhotos.SelectedItems.Count > 0) {
				ActivateButtons(true);
				if (lvPhotos.SelectedItems.Count == 1) {
					txtPhotoCaption.Text = lvPhotos.SelectedItems[0].SubItems[1].Text;
					txtPhotoCaption.Enabled = true;
					if (lvPhotos.SelectedItems[0].Text != "{Page Start}") {
						string strItem = lvPhotos.SelectedItems[0].Text;
						strItem = (System.IO.File.Exists(strItem) == true ? strItem : txtBaseFolder.Text + (txtBaseFolder.Text.EndsWith(System.IO.Path.DirectorySeparatorChar + "") ? "" : System.IO.Path.DirectorySeparatorChar + "") + strItem);
						if (System.IO.File.Exists(strItem) == true)
							strSelectedPhoto = strItem;
					}
				} else {
					txtPhotoCaption.Text = "";
					txtPhotoCaption.Enabled = false;
				}
				bChanged = true;
			} else {
				ActivateButtons(false);
				txtPhotoCaption.Text = "";
				txtPhotoCaption.Enabled = false;
				strSelectedPhoto = "";
			}
		}

		private void lvPhotos_DragEnter(object sender, DragEventArgs e) {
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
				e.Effect = DragDropEffects.Copy;
			else
				e.Effect = DragDropEffects.None;
		}

		private void lvPhotos_DragDrop(object sender, DragEventArgs e) {
			// find drag-drop position
			ListViewItem item = lvPhotos.GetItemAt(e.X - this.Location.X - groupPhotos.Location.X - lvPhotos.Location.X, e.Y - this.menuMain.Height - this.Location.Y - groupPhotos.Location.Y - lvPhotos.Location.Y);
			int iIndex = (item == null ? lvPhotos.Items.Count : lvPhotos.Items.IndexOf(item) + 1);
			// get dropped photos
			string[] astrFiles = (string[]) e.Data.GetData(DataFormats.FileDrop);
			lvPhotos.BeginUpdate();
			foreach (string strFileName in astrFiles) {
				string strAmendedFileName = strFileName;
				if ((txtBaseFolder.Text.Length > 0) && (strAmendedFileName.StartsWith(txtBaseFolder.Text)))
					strAmendedFileName = strAmendedFileName.Replace(txtBaseFolder.Text, "");
				lvPhotos.Items.Insert(iIndex++, new ListViewItem(new string[] { strAmendedFileName, "" }));
				bChanged = true;
			}
			lvPhotos.EndUpdate();
			lvPhotos.Refresh();
		}

		private void pbPreview_DoubleClick(object sender, EventArgs e) {
			if (strViewingPhoto != "")
				new SPACKImageDialog(strViewingPhoto).Show();
		}

		private void txtPhotoCaption_TextChanged(object sender, EventArgs e) {
			if (lvPhotos.SelectedItems.Count == 1) {
				lvPhotos.SelectedItems[0].SubItems[1].Text = txtPhotoCaption.Text;
				bChanged = true;
			}
		}

		private void timerUpdate_Tick(object sender, EventArgs e) {
			if (iLastLoadedPhoto < iLoadedPhoto) {  // update preview photo
				iLastLoadedPhoto = iLoadedPhoto;
				if (imagePreview != null) {  // adjust picture box image to correct size and dimensions
					int iMaxWidth = groupPhotoPreview.Width - 8;
					int iMaxHeight = groupPhotoPreview.Height - 23;
					if (imagePreview.Width > imagePreview.Height) {
						pbPreview.Left = 4;
						pbPreview.Width = iMaxWidth;
						pbPreview.Height = iMaxHeight * imagePreview.Height / imagePreview.Width;
						pbPreview.Top = 19 + (iMaxHeight - pbPreview.Height) / 2;
					} else {
						pbPreview.Top = 19;
						pbPreview.Height = iMaxHeight;
						pbPreview.Width = iMaxWidth * imagePreview.Width / imagePreview.Height;
						pbPreview.Left = 4 + (iMaxWidth - pbPreview.Width) / 2;
					}
					pbPreview.BackgroundImage = imagePreview;
					pbPreview.Refresh();
				} else {  // nullify
					pbPreview.BackgroundImage = null;
					pbPreview.Refresh();
				}
			}
		}

		#endregion

		#region Main album creation functions

		private void CreateAlbum(string strAlbumFolder) {

			// prepare initial parameters
			string strPageDescription = "";
			string strPhotosHTML = "";
			string strSavePath = strAlbumFolder + (strAlbumFolder.EndsWith(System.IO.Path.DirectorySeparatorChar + "") ? "" : System.IO.Path.DirectorySeparatorChar + "");
			bCreatingAlbum = true;
			strStatus = "Beginning creation of album " + txtAlbumTitle.Text + ".\r\n";

			// pop up report page on new background thread
			Thread thread = new Thread(new ThreadStart(ShowReport));
			thread.IsBackground = true;
			thread.Start();
			while (report == null)
				Thread.Sleep(10);

			// start
			if (System.IO.File.Exists(options_set.strHTMLTemplate) == false)
				strStatus += "Error: Couldn't find template file. Check your program settings.\r\n";
			else {
				// count pages
				int iTotalPages = 1;
				for (int iIndex = 1; (iIndex < lvPhotos.Items.Count) && (report != null) && (report.bAbort == false); iIndex++) {
					if (lvPhotos.Items[iIndex].SubItems[0].Text == "{Page Start}")
						iTotalPages++;
				}
				int iCurrentPage = 1;
				for (int iIndex = 0; (iIndex < lvPhotos.Items.Count) && (report != null) && (report.bAbort == false); iIndex++) {
					string strItem = lvPhotos.Items[iIndex].SubItems[0].Text;
					string strItemDescription = lvPhotos.Items[iIndex].SubItems[1].Text;
					if (strItem == "{Page Start}") {  // write previous page
						if (iIndex != 0) {
							strStatus += WriteIndex(strPageDescription, strPhotosHTML, iCurrentPage, iTotalPages, strAlbumFolder);
							iCurrentPage++;
						}
						strPageDescription = strItemDescription;
						strPhotosHTML = "";
					} else {  // copy image
						try {
							string strFilename = (System.IO.File.Exists(strItem) == true ? strItem : txtBaseFolder.Text + (txtBaseFolder.Text.EndsWith(System.IO.Path.DirectorySeparatorChar + "") ? "" : System.IO.Path.DirectorySeparatorChar + "") + strItem);
							if (System.IO.File.Exists(strFilename) == false)
								strStatus += "Error: Couldn't find image " + strItem + ". (Check the base path for this album.)\r\n";
							else {
								Bitmap bitmap = (options_set.strResampleMode == "Percent" ? ResampleImage_Percent(strFilename, options_set.iResampleImagesValue) : ResampleImage_Absolute(strFilename, options_set.iResampleImagesValue));
								bitmap.Save(strSavePath + System.IO.Path.GetFileName(strItem));
								bitmap.Dispose();
								strPhotosHTML += "<IMG SRC=\"" + System.IO.Path.GetFileName(strItem) + "\" /><P />" + (strItemDescription.Length > 0 ? strItemDescription + "<P />" : "");
								strStatus += "Copied image " + System.IO.Path.GetFileName(strItem) + ".\r\n";
							}
						} catch (Exception ex) {
							strStatus += "Error converting image " + strItem + ": " + ex.Message + "\r\n";
						}
					}
				}
				if ((report != null) && (report.bAbort == false))
					strStatus += WriteIndex(strPageDescription, strPhotosHTML, iCurrentPage, iTotalPages, strAlbumFolder);
				
				// notify report class that album is done
				strStatus += (((report == null) || (report.bAbort == false)) ? "Done.\r\n" : "Aborted.\r\n");
			}
			bCreatingAlbum = false;
		}

		public string WriteIndex(string strPageDescription, string strPhotosHTML, int iCurrentPage, int iTotalPages, string strAlbumFolder) {
			try {
				StreamReader s = new StreamReader(options_set.strHTMLTemplate);
				string strIndex = s.ReadToEnd();
				s.Close();
				strIndex = strIndex.Replace("<SPACK_Album_Title />", txtAlbumTitle.Text);
				strIndex = strIndex.Replace("<SPACK_Album_Description />", txtAlbumDescription.Text);
				strIndex = strIndex.Replace("<SPACK_Page_Description />", strPageDescription);
				strIndex = strIndex.Replace("<SPACK_Images />", strPhotosHTML);
				if (iCurrentPage == 1)
					strIndex = strIndex.Replace("<SPACK_Previous_Index />", "");
				else if (iCurrentPage == 2)
					strIndex = strIndex.Replace("<SPACK_Previous_Index />", "<A HREF=\"index.html\">Back</A>");
				else
					strIndex = strIndex.Replace("<SPACK_Previous_Index />", "<A HREF=\"index" + (iCurrentPage - 1) + ".html\">Back</A>");
				if (iCurrentPage < iTotalPages)
					strIndex = strIndex.Replace("<SPACK_Next_Index />", "<A HREF=\"index" + (iCurrentPage + 1) + ".html\">Next</A>");
				else
					strIndex = strIndex.Replace("<SPACK_Next_Index />", "");
				StreamWriter w = new StreamWriter(strAlbumFolder + (strAlbumFolder.EndsWith(System.IO.Path.DirectorySeparatorChar + "") ? "" : System.IO.Path.DirectorySeparatorChar + "") + "index" + (iCurrentPage == 1 ? ".html" : iCurrentPage.ToString() + ".html"), false);
				w.Write(strIndex);
				w.Close();
				return "Wrote index page " + iCurrentPage + ".\r\n";
			} catch (Exception ex) {
				return "Error: " + ex.Message;
			}
		}

		#endregion

		#region Helper functions

		private void CreateNewAlbum() {
			// create new album
			txtAlbumTitle.Text = "";
			txtAlbumDescription.Text = "";
			txtBaseFolder.Text = options_set.strDefaultBasePath;
			txtPhotoCaption.Text = "";
			SetFilename("");
			strSelectedPhoto = "";
			strStatus = "";
			lvPhotos.Items.Clear();
			lvPhotos.Items.Add(new ListViewItem(new string[] { "{Page Start}", "" }));
			bChanged = false;
			ActivateButtons(false);
		}

		private void SaveAlbum(string strFilename) {
			// generate SPACK_Album object
			SPACK_Album album = new SPACK_Album();
			album.strAlbumTitle = txtAlbumTitle.Text;
			album.strAlbumDescription = txtAlbumDescription.Text;
			album.strBaseFolder = txtBaseFolder.Text;
			foreach (ListViewItem item in lvPhotos.Items)
				album.arrayPhotos.Add(new string[] { item.SubItems[0].Text, item.SubItems[1].Text });
			// serialize SPACK_Album
			album.SaveAlbum(strFilename);
			SetFilename(strFilename);
		}

		private void LoadAlbum(string strFilename) {
			// load SPACK_Album object
			SPACK_Album album = SPACK_Album.LoadAlbum(strFilename);
			if (album != null) {
				// unload data into form
				txtAlbumTitle.Text = album.strAlbumTitle;
				txtAlbumDescription.Text = album.strAlbumDescription;
				txtBaseFolder.Text = album.strBaseFolder;
				lvPhotos.BeginUpdate();
				lvPhotos.Items.Clear();
				foreach (string[] strEntry in album.arrayPhotos)
					lvPhotos.Items.Add(new ListViewItem(new string[] { strEntry[0], strEntry[1] }));
				lvPhotos.EndUpdate();
				SetFilename(strFilename);
			}
		}

		public static Bitmap ResampleImage_Percent(string strFilename, int iPercentage) {
			System.IO.FileStream instream = new System.IO.FileStream(strFilename, System.IO.FileMode.Open, System.IO.FileAccess.Read);
			Image image = Image.FromStream(instream);
			instream.Close();
			int iDownsampleHeight = (int) (image.Height * iPercentage / 100);
			int iDownsampleWidth = (int) (image.Width * iPercentage / 100);
			System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(iDownsampleWidth, iDownsampleHeight, image.PixelFormat);
			System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(bitmap);
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
			g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
			g.DrawImage(image, 0, 0, iDownsampleWidth, iDownsampleHeight);
			// clean up
			g.Dispose();
			image.Dispose();
			instream.Close();
			return bitmap;
		}

		public static Bitmap ResampleImage_Absolute(string strFilename, int iAbsolute) {
			System.IO.FileStream instream = new System.IO.FileStream(strFilename, System.IO.FileMode.Open, System.IO.FileAccess.Read);
			Image image = Image.FromStream(instream);
			instream.Close();
			int iDownsampleHeight = image.Height;
			int iDownsampleWidth = image.Width;
			if (iDownsampleHeight > iAbsolute) {
				iDownsampleWidth = iDownsampleWidth * iAbsolute / iDownsampleHeight;
				iDownsampleHeight = iAbsolute;
			}
			if (iDownsampleWidth > iAbsolute) {
				iDownsampleHeight = iDownsampleHeight * iAbsolute / iDownsampleWidth;
				iDownsampleWidth = iAbsolute;
			}
			System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(iDownsampleWidth, iDownsampleHeight, image.PixelFormat);
			System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(bitmap);
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
			g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
			g.DrawImage(image, 0, 0, iDownsampleWidth, iDownsampleHeight);
			// clean up
			g.Dispose();
			image.Dispose();
			instream.Close();
			return bitmap;
		}

		public void ActivateButtons(bool bActivate) {
			btnRemove.Enabled = btnMoveDown.Enabled = btnMoveUp.Enabled = bActivate;
		}

		public void SetFilename(string strFilename) {
			strAlbumFilename = strFilename;
			this.Text = "Simple Photo Album Creation Kit (SPACK)" + (strFilename.Length == 0 ? "" : ": " + System.IO.Path.GetFileNameWithoutExtension(strFilename));
		}

		public void ShowReport() {
			report = new SPACK_Report(this);
			report.ShowDialog();
		}


		private void SortPhotos(string strSortType) {
			// This function sorts the photo list by one of these categories: Name, Date Created, Date Updated.
			lvPhotos.BeginUpdate();
			List<string[]> arrayPhotos = new List<string[]>();
			int iStart = 0;
			for (int i = 0; i <= lvPhotos.Items.Count; i++) {
				if ((i == lvPhotos.Items.Count) || ((lvPhotos.Items[i]).SubItems[0].Text == "{Page Start}")) {
					// sort photos in current set
					if (arrayPhotos.Count > 0) {
						// remove range of photos
						for (int j = 0; j < arrayPhotos.Count; j++)
							lvPhotos.Items.RemoveAt(iStart);
						// sort photos
						arrayPhotos.Sort(new StringComparer());
						// add in reverse order
						while (arrayPhotos.Count > 0) {
							lvPhotos.Items.Insert(iStart, new ListViewItem(new string[] { arrayPhotos[arrayPhotos.Count - 1][1], arrayPhotos[arrayPhotos.Count - 1][2] }));
							arrayPhotos.RemoveAt(arrayPhotos.Count - 1);
						}
					}
					// prepare for next set
					arrayPhotos.Clear();
					iStart = i + 1;
				} else {
					string strFilename = lvPhotos.Items[i].SubItems[0].Text;
					if (System.IO.File.Exists(strFilename) == false)
						strFilename = txtBaseFolder.Text + (txtBaseFolder.Text.EndsWith(System.IO.Path.DirectorySeparatorChar + "") ? "" : System.IO.Path.DirectorySeparatorChar + "") + strFilename;
					if (System.IO.File.Exists(strFilename)) {
						switch (strSortType) {
							case "Name":
								arrayPhotos.Add(new string[] { System.IO.Path.GetFileNameWithoutExtension(lvPhotos.Items[i].SubItems[0].Text), lvPhotos.Items[i].SubItems[0].Text, lvPhotos.Items[i].SubItems[1].Text });
								break;
							case "Date Created":
								DateTime dtDateCreated = System.IO.File.GetCreationTime(strFilename);
								arrayPhotos.Add(new string[] { dtDateCreated.ToString("u"), lvPhotos.Items[i].SubItems[0].Text, lvPhotos.Items[i].SubItems[1].Text });
								break;
							case "Date Modified":
								DateTime dtDateModified = System.IO.File.GetLastWriteTime(strFilename);
								arrayPhotos.Add(new string[] { dtDateModified.ToString("u"), lvPhotos.Items[i].SubItems[0].Text, lvPhotos.Items[i].SubItems[1].Text });
								break;
							default:
								break;
						}
					}
				}
			}
			lvPhotos.EndUpdate();
			lvPhotos.Refresh();
		}

		#endregion

		#region Image loading thread function

		// SPACK uses a separate thread to monitor the selection of images in the textbox, and to load them into the application in the
		// background. This loading can be surprisingly CPU-intensive for even mid-size images (5mb), and foreground loading was bogging
		// down the user interface. So I spun off this step as a separate thread, and it works much better. The image loads in the
		// background, and the thread signals the main thread to update its image via bUpdatePhoto. This is monitored by the timer
		// function on the main thread, which loads the image into the picture box. - Probably not the most efficient mechanism in the
		// world, but it works.

		private void LoadImagesThread() {
			while (true) {
				if (strSelectedPhoto != strViewingPhoto) {
					strViewingPhoto = strSelectedPhoto;
					if (strSelectedPhoto.Length > 0) {
						System.IO.FileStream stream = new System.IO.FileStream(strSelectedPhoto, System.IO.FileMode.Open);
						imagePreview = Image.FromStream(stream);
						stream.Close();
					} else
						imagePreview = null;
					iLoadedPhoto++;
				}
				Thread.Sleep(100);
			}
		}

		#endregion

	}

	#region Helper classes

	[Serializable]
	public class SPACK_Options_Set {

		#region Variables

		public string strHTMLTemplate = "";
		public string strDefaultBasePath = "";
		public string strResampleMode = "Percent";
		public int iResampleImagesValue = 100;

		public SPACK_Options_Set() {
			strHTMLTemplate = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location) + System.IO.Path.DirectorySeparatorChar + "SPACK Template.html";
		}

		#endregion

	}

	public class SortedItem {

		#region Variables and initialization code

		public int iNewIndex;
		public ListViewItem item;

		public SortedItem(int iNewIndex, ListViewItem item) {
			this.iNewIndex = iNewIndex;
			this.item = item;
		}

		#endregion

	}

	public class StringComparer : IComparer<string[]> {

		#region Helper function

		public int Compare(string[] x, string[] y) {
			if ((y == null) || (y.Length < 1))
				return 0;
			else if ((x == null) || (x.Length < 1))
				return 1;
			int i = String.Compare(x[0], y[0]);
			if (i < 0)
				return -1;
			else if (i > 0)
				return 1;
			else
				return 0;
		}

		#endregion
	
	}

	[Serializable]
	public class SPACK_Album {

		#region Variables

		public string strAlbumTitle = "";
		public string strAlbumDescription = "";
		public string strBaseFolder = "";
		public List<string[]> arrayPhotos = new List<string[]>();

		#endregion

		#region Load and Save functions

		public static SPACK_Album LoadAlbum(string strFilename) {
			SPACK_Album album = null;
			try {
				if (System.IO.File.Exists(strFilename) == true) {
					XmlSerializer formatter = new XmlSerializer(typeof(SPACK_Album));
					TextReader file = new StreamReader(strFilename);
					album = (SPACK_Album) formatter.Deserialize(file);
					file.Close();
				}
			} catch (Exception ex) {
				MessageBox.Show(ex.Message);
			}
			return album;
		}

		public void SaveAlbum(string strFilename) {
			try {
				XmlSerializer formatter = new XmlSerializer(typeof(SPACK_Album));
				TextWriter file = new StreamWriter(strFilename, false);
				formatter.Serialize(file, this);
				file.Close();
			} catch (Exception ex) {
				MessageBox.Show(ex.Message);
			}
		}

		#endregion

	}

	#endregion

}
